/*
babyphone - A baby monitor application on the Nokia N900.
    Copyright (C) 2010  Roman Morawek <maemo@morawek.at>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>
#include <QDBusMessage>

#include "audiomonitor.h"
#include "callmonitor.h"
#include "usernotifier.h"
#include "audiolevelgraphicsscene.h"
#include "settings.h"
#include "profileswitcher.h"


namespace Ui {
    class MainWindow;
}


/*!
  MainWindow is the main application class that handles the UI interaction and 
  holds references to the functionality classes.
  Initially it sets up the user interface and starts the audio capturing. It 
  instantiates the Settings as well as the UserNotifier.
*/
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    void activateMonitor();
    void deactivateMonitor();
    void showFirstRunInfo() const;
    void showNotificationError() const;

private slots:
    void refreshAudioData(int counter, int value);
    void notifyFinished();
    void startAudio();
    void stopAudio();
    void activationTimerExpired();
    void bringWindowToFront();
    void displayDimmed(const QDBusMessage&);

    void on_pushButton_clicked();
    void on_pushButton_contacts_clicked();
    void on_pushButton_enterNumber_clicked();
    void on_actionAbout_triggered();


private:
    //! autogenerated user interface
    Ui::MainWindow *ui;			
	
    //! the graphics item holding the volume audio chart
    AudioLevelGraphicsScene *itsAudioLevelGraphics;

    //! the graphics item holding the time related audio threshold counter
    AudioLevelGraphicsScene *itsAudioCounterGraphics;

    //! master reference to global application settings
    Settings *itsSettings;

    //! the audio monitor functionality
    AudioMonitor *itsAudioMonitor;

    //! the monitor checking incoming call and call status
    CallMonitor *itsCallMonitor;

    //! the phone handler notifying the parents on triggering
    UserNotifier *itsUserNotifier;

    //! the profile switcher to disable the ringtones
    ProfileSwitcher *itsProfileSwitcher;


    //! application status: if true, an audio event triggers the phone call
    bool itsMonitorActive;

    //! application status: if true the application is actually shown on the screen
    bool itsIsScreenOff;

    //! timer for delayed activation
    QTimer itsActivationDelayTimer;

    //! indicates an active notification call. During that time audio events are ignored
    bool itsNotificationPending;
};


#endif // MAINWINDOW_H
